DECLARE FUNCTION GetTemp! (Unit$, Emiss!, PollAddr%, COMno%)

CLS
PRINT "              QBASIC Example Program for Reading Temperatures"
PRINT " ==========================================================================="
PRINT
'This program uses only one function call. It starts by initializes
'the COM-port, reading the device's ID and setting the emissivity.
'Also, it will set the emissivity each time as it is changed.
'The function will print error and warning messages and set the
'temperature value to 100000 degrees. (Normally, this does not occur).
'The only thing the main program needs to do is to call this function
'with the correct parameters (emissivity, polling address and COM-No).
'The function can be handled as a "black box" for those that just want
'to get the temperature back from the sensor.The returned values are
'the object temperature and its units (deg C or deg F) in string Unit$.
'To set the emissivity in this example, simply press the "E" key while
'the program is running and enter a new value. To stop the program press
'the SPACEBAR or ESC.

Emiss = .95
DO
  Temp = GetTemp(Unit$, Emiss, 0, 1)           'PollAddr=0, COMno=1
  IF Temp < 10000 THEN
    PRINT USING "###.#"; Temp; : PRINT Unit$
    in$ = INKEY$
    IF (in$ = "e") OR (in$ = "E") THEN : INPUT "New Emissivity=", Emiss
  END IF
LOOP WHILE (in$ <> " ") AND (in$ <> CHR$(&H1B)) AND (Temp < 10000)

'============================================================================

FUNCTION GetTemp (U$, Emiss!, PollAddr%, COMno%) STATIC
  CONST SlaveTimeOut = &H100, FrameError = &H200
  CONST CheckSumError = &H300, ResponseError = &H400
  DIM TxBuffer(100), RxBuffer(100) AS INTEGER
  DIM A(15, 4, 5), C(4), E(15, 4), buffer AS STRING * 4
  DEFINT B, I, P
  DEF SEG = 0
  GetTemp = 100000
  IF C(COMno%) = 0 THEN
    PRINT "Init COM"; COMno%
    COMmemAddr% = &H400 + (COMno% - 1) * 2
    COMaddr% = PEEK(COMmemAddr%) + 256 * PEEK(COMmemAddr% + 1)
    IF COMaddr% = 0 THEN
      PRINT "COM"; COMno%; "not available !"
      EXIT FUNCTION
    END IF
    OUT COMaddr% + 3, &H80
    OUT COMaddr%, 96
    OUT COMaddr% + 1, 0
    OUT COMaddr% + 3, &HB
    OUT COMaddr% + 1, 0
    OUT COMaddr% + 4, 1
    C(COMno%) = COMaddr%
  ELSE : COMaddr% = C(COMno%)
  END IF
  IF A(PollAddr%, COMno%, 0) = 0 THEN
    TxBuffer(0) = 2
    TxBuffer(1) = PollAddr%
    TxBuffer(2) = 0
    TxBuffer(3) = 0
    TxCount = 4
    GOSUB Transfer
    IF TransferError <> 0 THEN : EXIT FUNCTION
    A(PollAddr%, COMno%, 0) = RxBuffer(DataPos + 1) AND &H3F
    A(PollAddr%, COMno%, 1) = RxBuffer(DataPos + 2)
    A(PollAddr%, COMno%, 2) = RxBuffer(DataPos + 9)
    A(PollAddr%, COMno%, 3) = RxBuffer(DataPos + 10)
    A(PollAddr%, COMno%, 4) = RxBuffer(DataPos + 11)
    DevID& = RxBuffer(DataPos + 9)
    DevID& = DevID& * 256 + RxBuffer(DataPos + 10)
    DevID& = DevID& * 256 + RxBuffer(DataPos + 11)
    PRINT "Device ID:"; DevID&
  END IF
  IF E(PollAddr%, COMno%) <> Emiss! THEN
    TxBuffer(0) = &H82
    FOR i = 0 TO 4: TxBuffer(i + 1) = A(PollAddr%, COMno%, i): NEXT
    TxBuffer(6) = 129: TxBuffer(7) = 4: TxCount = 8
    s = Emiss!
    GOSUB WriteSingle
    GOSUB Transfer
    IF (TransferError OR DeviceError) <> 0 THEN : EXIT FUNCTION
    p = DataPos: GOSUB ReadSingle: E(PollAddr%, COMno%) = CINT(s * 100) / 100
    PRINT USING "Emissivity= #.##"; E(PollAddr%, COMno%)
  END IF
  TxBuffer(0) = &H82
  FOR i = 0 TO 4: TxBuffer(i + 1) = A(PollAddr%, COMno%, i): NEXT
  TxBuffer(6) = 1: TxBuffer(7) = 0: TxCount = 8
  GOSUB Transfer
  IF TransferError <> 0 THEN EXIT FUNCTION
  UnitCode = RxBuffer(DataPos): p = DataPos + 1: GOSUB ReadSingle
  GetTemp = s
  U$ = ""
  IF UnitCode = 32 THEN : U$ = "C"
  IF UnitCode = 33 THEN : U$ = "F"
  EXIT FUNCTION

GetTime:
  DEF SEG = 0
  Time& = PEEK(&H46F) * 256 + PEEK(&H46E)
  Time& = (Time& * 256 + PEEK(&H46D)) * 256 + PEEK(&H46C)
RETURN

SendByte:
  WHILE (INP(COMaddr% + 5) AND &H40) = 0
  WEND
  OUT COMaddr%, b
RETURN

ReceiveByte:
  GOSUB GetTime: ByteTime& = Time&
  DO: GOSUB GetTime: dt& = Time& - ByteTime&
  LOOP WHILE ((INP(COMaddr% + 5) AND 1) = 0) AND ((INP(COMaddr% + 6) AND &H80) <> 0) AND (dt < 2)
  b = INP(COMaddr%)
  IF dt& >= 2 THEN TransferError = SlaveTimeOut
  TransferError = TransferError OR (INP(COMaddr% + 4) AND &H1E)
RETURN

Transfer:
  Try = 0
  DO
    TransferError = 0
    BusFreeTime& = Time&
    DO: GOSUB GetTime
      IF (INP(COMaddr% + 6) AND &H80) <> 0 THEN : BusFreeTime& = Time&
    LOOP WHILE (Time& - BusFreeTime&) < 2
    OUT COMaddr% + 4, 3
    b = &HFF
    FOR i = 1 TO 3: GOSUB SendByte: NEXT
    CheckByte = 0
    FOR i = 0 TO TxCount - 1
      b = TxBuffer(i): CheckByte = CheckByte XOR b: GOSUB SendByte
    NEXT
    b = CheckByte: GOSUB SendByte
    WHILE (INP(COMaddr% + 5) AND &H60) <> &H60: WEND
    OUT COMaddr% + 4, 1
    WHILE (DCD <> 0): WEND
    GOSUB GetTime: AnswerTime& = Time&
    WHILE ((INP(COMaddr% + 6) AND &H80) = 0) AND (TransferError = 0)
      GOSUB GetTime
      IF (Time& - AnswerTime&) >= 5 THEN : TransferError = SlaveTimeOut
    WEND
    IF TransferError = 0 THEN
      DO: DO: GOSUB ReceiveByte
        IF ((INP(COMaddr% + 6) AND &H80) = 0) THEN
          TransferError = SlaveTimeOut
          EXIT FUNCTION
        END IF
        TransferError = TransferError AND &HFFFFFFF0
        LOOP WHILE b <> &HFF
        DO: GOSUB ReceiveByte
        IF ((INP(COMaddr% + 6) AND &H80) = 0) THEN
          TransferError = SlaveTimeOut
          EXIT FUNCTION
        END IF
        LOOP WHILE b = &HFF
      LOOP WHILE (b <> &H6) AND (b <> &H86)
      RxBuffer(0) = b
      RxCount = 1
      WHILE (INP(COMaddr% + 6) AND &H80) <> 0
        GOSUB ReceiveByte
        RxBuffer(RxCount) = b
        RxCount = RxCount + 1
      WEND
      IF TransferError = 0 THEN
        IF RxBuffer(0) = &H6 THEN : ByteCountPos = 3
        IF RxBuffer(0) = &H86 THEN : ByteCountPos = 7
        FrameEndPos = RxBuffer(ByteCountPos) + ByteCountPos + 1
        IF RxCount < FrameEndPos THEN
          TransferError = FrameError
        ELSE
          CheckByte = 0
          FOR i = 0 TO FrameEndPos: CheckByte = CheckByte XOR RxBuffer(i): NEXT
          IF CheckByte <> 0 THEN
            TransferError = CheckSumError
          ELSE
            DeviceError = RxBuffer(ByteCountPos + 1)
            DeviceStatus = RxBuffer(ByteCountPos + 2)
            DataPos = ByteCountPos + 3
            IF DeviceError > &H80 THEN : TransferError = ResponseError
          END IF
        END IF
      END IF
    END IF
    Try = Try + 1
  LOOP WHILE (Try <= 3) AND (TransferError <> 0)
 
  IF TransferError <> 0 THEN
    PRINT "Communication Error: ";
    IF (TransferError AND &HF) <> 0 THEN
      PRINT "UART receive errors"
    ELSEIF TransferError = SlaveTimeOut THEN
      PRINT "Slave Timeout"
    ELSEIF TransferError = FrameError THEN
      PRINT "Frame Error"
    ELSEIF TransferError = CheckSumError THEN
      PRINT "Checksum Error"
    ELSEIF TransferError = ResponseError THEN
      PRINT "Device Error Code: "; HEX$(DeviceError)
    END IF
 
  ELSEIF DeviceError <> 0 THEN
    PRINT "Device Warning Code:"; DeviceError;
    IF DeviceError = 2 THEN
      PRINT ": Invalid Selection"
    ELSEIF DeviceError = 3 THEN
      PRINT ": Passed parameter to large"
    ELSEIF DeviceError = 4 THEN
      PRINT ": Passed parameter to small"
    ELSEIF DeviceError = 5 THEN
      PRINT ": Too few data bytes received"
    ELSEIF DeviceError = 7 THEN
      PRINT ": Device in write-protect mode"
    ELSEIF DeviceError = 8 THEN
      PRINT ": Set to nearest possible value"
    ELSEIF DeviceError = 9 THEN
      PRINT ": Lower range value too high"
    ELSEIF DeviceError = 10 THEN
      PRINT ": Lower range value too low"
    ELSEIF DeviceError = 11 THEN
      PRINT ": Upper range value too high"
    ELSEIF DeviceError = 12 THEN
      PRINT ": Upper range value too low"
    ELSEIF DeviceError = 14 THEN
      PRINT ": Span too small"
    ELSEIF DeviceError = 32 THEN
      PRINT ": Device busy"
    ELSEIF DeviceError = 64 THEN
      PRINT ": Command not implemented"
    PRINT
    END IF
  END IF
RETURN

WriteSingle:
  buffer = MKS$(s)
  TxBuffer(TxCount) = ASC(MID$(buffer, 4, 1))
  TxBuffer(TxCount + 1) = ASC(MID$(buffer, 3, 1))
  TxBuffer(TxCount + 2) = ASC(MID$(buffer, 2, 1))
  TxBuffer(TxCount + 3) = ASC(MID$(buffer, 1, 1))
  TxCount = TxCount + 4
RETURN

ReadSingle:
  MID$(buffer, 1, 1) = CHR$(RxBuffer(p + 3))
  MID$(buffer, 2, 1) = CHR$(RxBuffer(p + 2))
  MID$(buffer, 3, 1) = CHR$(RxBuffer(p + 1))
  MID$(buffer, 4, 1) = CHR$(RxBuffer(p))
  s = CVS(buffer)
RETURN

END FUNCTION

